@@ -22,7 +22,9 @@ import getpass
 import inspect
 import socket
 import threading
+import datetime
 import time
+import uuid
 import warnings
 import weakref
 import zmq
@@ -33,6 +35,14 @@ from kitchen.text.converters import to_bytes
 import fedmsg.encoding
 import fedmsg.crypto
 
+from fedmsg.utils import (
+    set_high_water_mark,
+    guess_calling_module,
+    set_tcp_keepalive,
+)
+
+from fedmsg.replay import check_for_replay
+
 import logging
 
 
@@ -49,13 +59,12 @@ class FedMsgContext(object):
 
         # Prepare our context and publisher
         self.context = zmq.Context(config['io_threads'])
-        method = config.get('active', False) or 'bind' and 'connect'
         method = ['bind', 'connect'][config['active']]
 
         # If no name is provided, use the calling module's __name__ to decide
         # which publishing endpoint to use.
         if not config.get("name", None):
-            module_name = self.guess_calling_module(default="fedmsg")
+            module_name = guess_calling_module(default="fedmsg")
             config["name"] = module_name + '.' + self.hostname
 
             if any(map(config["name"].startswith, ['fedmsg'])):
@@ -63,14 +72,17 @@ class FedMsgContext(object):
 
         # Find my message-signing cert if I need one.
         if self.c.get('sign_messages', False) and config.get("name"):
-            if 'cert_prefix' in config:
-                cert_index = "%s.%s" % (config['cert_prefix'], self.hostname)
-            else:
-                cert_index = config['name']
-                if cert_index == 'relay_inbound':
-                    cert_index = "shell.%s" % self.hostname
+            if not config.get("crypto_backend") == "gpg":
+                if 'cert_prefix' in config:
+                    cert_index = "%s.%s" % (config['cert_prefix'], self.hostname)
+                else:
+                    cert_index = config['name']
+                    if cert_index == 'relay_inbound':
+                        cert_index = "shell.%s" % self.hostname
 
-            self.c['certname'] = self.c['certnames'][cert_index]
+                self.c['certname'] = self.c['certnames'][cert_index]
+            else:
+                self.c['gpg_signing_key'] = self.c['gpg_keys'][cert_index]
 
         # Do a little special-case mangling.  We never want to "listen" to the
         # relay_inbound address, but in the special case that we want to emit
@@ -93,8 +105,8 @@ class FedMsgContext(object):
             # Construct it.
             self.publisher = self.context.socket(zmq.PUB)
 
-            self._set_high_water_mark(self.publisher)
-            self._set_tcp_keepalive(self.publisher)
+            set_high_water_mark(self.publisher, config)
+            set_tcp_keepalive(self.publisher, config)
 
             # Set a zmq_linger, thus doing a little bit more to ensure that our
             # message gets to the fedmsg-relay (*if* we're talking to the relay
@@ -136,14 +148,18 @@ class FedMsgContext(object):
             # connection, then there are not enough endpoints listed in the
             # config for the number of processes attempting to use fedmsg.
             if not _established:
-                raise IOError("Couldn't find an available endpoint.")
+                raise IOError(
+                    "Couldn't find an available endpoint "
+                    "for name %r" % config.get("name", None))
 
         elif config.get('mute', False):
             # Our caller doesn't intend to send any messages.  Pass silently.
             pass
         else:
             # Something is wrong.
-            warnings.warn("fedmsg is not configured to send any messages")
+            warnings.warn(
+                "fedmsg is not configured to send any messages "
+                "for name %r" % config.get("name", None))
 
         # Cleanup.  See http://bit.ly/SaGeOr for discussion.
         weakref.ref(threading.current_thread(), self.destroy)
@@ -152,52 +168,6 @@ class FedMsgContext(object):
         # tries anything.  This is a documented zmq 'feature'.
         time.sleep(config['post_init_sleep'])
 
-    def _set_tcp_keepalive(self, socket):
-        """ Set a series of TCP keepalive options on the socket if
-        and only if
-          1) they are specified explicitly in the config and
-          2) the version of pyzmq has been compiled with support
-
-        We ran into a problem in FedoraInfrastructure where long-standing
-        connections between some hosts would suddenly drop off the
-        map silently.  Because PUB/SUB sockets don't communicate
-        regularly, nothing in the TCP stack would automatically try and
-        fix the connection.  With TCP_KEEPALIVE options (introduced in
-        libzmq 3.2 and pyzmq 2.2.0.1) hopefully that will be fixed.
-
-        See the following
-          - http://tldp.org/HOWTO/TCP-Keepalive-HOWTO/overview.html
-          - http://api.zeromq.org/3-2:zmq-setsockopt
-        """
-
-        keepalive_options = {
-            # Map fedmsg config keys to zeromq socket constants
-            'zmq_tcp_keepalive': 'TCP_KEEPALIVE',
-            'zmq_tcp_keepalive_cnt': 'TCP_KEEPALIVE_CNT',
-            'zmq_tcp_keepalive_idle': 'TCP_KEEPALIVE_IDLE',
-            'zmq_tcp_keepalive_intvl': 'TCP_KEEPALIVE_INTVL',
-        }
-        for key, const in keepalive_options.items():
-            if key in self.c:
-                attr = getattr(zmq, const, None)
-                if attr:
-                    self.log.debug("Setting %r %r" % (const, self.c[key]))
-                    socket.setsockopt(attr, self.c[key])
-
-    def _set_high_water_mark(self, socket):
-        """ Set a high water mark on the zmq socket.  Do so in a way that is
-        cross-compatible with zeromq2 and zeromq3.
-        """
-
-        if self.c['high_water_mark']:
-            if hasattr(zmq, 'HWM'):
-                # zeromq2
-                socket.setsockopt(zmq.HWM, self.c['high_water_mark'])
-            else:
-                # zeromq3
-                socket.setsockopt(zmq.SNDHWM, self.c['high_water_mark'])
-                socket.setsockopt(zmq.RCVHWM, self.c['high_water_mark'])
-
     def destroy(self):
         """ Destroy a fedmsg context """
 
@@ -209,20 +179,8 @@ class FedMsgContext(object):
             self.context.term()
             self.context = None
 
-    # TODO -- this should be in kitchen, not fedmsg
-    def guess_calling_module(self, default=None):
-        # Iterate up the call-stack and return the first new top-level module
-        for frame in (f[0] for f in inspect.stack()):
-            modname = frame.f_globals['__name__'].split('.')[0]
-            if modname != "fedmsg":
-                return modname
-
-        # Otherwise, give up and just return the default.
-        return default
-
     def send_message(self, topic=None, msg=None, modname=None):
-        warnings.warn(".send_message is deprecated.",
-                      warnings.DeprecationWarning)
+        warnings.warn(".send_message is deprecated.", DeprecationWarning)
 
         return self.publish(topic, msg, modname)
 
@@ -235,7 +193,7 @@ class FedMsgContext(object):
           ... })
 
         The above snippet will send the message ``'{test: "Hello World"}'``
-        over the ``org.fedoraproject.dev.test.testing`` topic.
+        over the ``<topic_prefix>.dev.test.testing`` topic.
 
         This function (and other API functions) do a little bit more
         heavy lifting than they let on.  If the "zeromq context" is not yet
@@ -296,7 +254,7 @@ class FedMsgContext(object):
         msg = msg or dict()
 
         # If no modname is supplied, then guess it from the call stack.
-        modname = modname or self.guess_calling_module(default="fedmsg")
+        modname = modname or guess_calling_module(default="fedmsg")
         topic = '.'.join([modname, topic])
 
         if topic[:len(self.c['topic_prefix'])] != self.c['topic_prefix']:
@@ -309,11 +267,14 @@ class FedMsgContext(object):
         if type(topic) == unicode:
             topic = to_bytes(topic, encoding='utf8', nonstring="passthru")
 
+        year = datetime.datetime.now().year
+
         self._i += 1
         msg = dict(
             topic=topic,
             msg=msg,
             timestamp=time.time(),
+            msg_id=str(year) + '-' + str(uuid.uuid4()),
             i=self._i,
             username=getpass.getuser(),
         )
@@ -321,6 +282,11 @@ class FedMsgContext(object):
         if self.c.get('sign_messages', False):
             msg = fedmsg.crypto.sign(msg, **self.c)
 
+        store = self.c.get('persistent_store', None)
+        if store:
+            # Add the seq_id field
+            msg = store.add(msg)
+
         self.publisher.send_multipart(
             [topic, fedmsg.encoding.dumps(msg)],
             flags=zmq.NOBLOCK,
@@ -343,7 +309,10 @@ class FedMsgContext(object):
 
         failed_hostnames = []
         subs = {}
+        watched_names = {}
         for _name, endpoint_list in self.c['endpoints'].iteritems():
+            # Listify endpoint_list in case it is a single string
+            endpoint_list = iterate(endpoint_list)
             for endpoint in endpoint_list:
                 # First, some sanity checking.  zeromq will potentially
                 # segfault if we don't do this check.
@@ -363,11 +332,14 @@ class FedMsgContext(object):
                 subscriber = self.context.socket(zmq.SUB)
                 subscriber.setsockopt(zmq.SUBSCRIBE, topic)
 
-                self._set_high_water_mark(subscriber)
-                self._set_tcp_keepalive(subscriber)
+                set_high_water_mark(subscriber, self.c)
+                set_tcp_keepalive(subscriber, self.c)
 
                 getattr(subscriber, method)(endpoint)
                 subs[subscriber] = (_name, endpoint)
+            if _name in self.c.get("replay_endpoints", {}):
+                # At first we don't know where the sequence is at.
+                watched_names[_name] = -1
 
         # Register the sockets we just built with a zmq Poller.
         poller = zmq.Poller()
@@ -385,10 +357,23 @@ class FedMsgContext(object):
                     _name, ep = subs[s]
                     _topic, message = s.recv_multipart()
                     msg = fedmsg.encoding.loads(message)
-                    if not validate:
-                        yield _name, ep, _topic, msg
-                    elif fedmsg.crypto.validate(msg, **self.c):
-                        yield _name, ep, _topic, msg
+                    if not validate or fedmsg.crypto.validate(msg, **self.c):
+                        # If there is even a slight change of replay, use
+                        # check_for_replay
+                        if len(self.c.get('replay_endpoints', {})) > 0:
+                            for m in check_for_replay(
+                                    _name, watched_names,
+                                    msg, self.c, self.context):
+
+                                # Revalidate all the replayed messages.
+                                if not validate or \
+                                        fedmsg.crypto.validate(m, **self.c):
+                                    yield _name, ep, m['topic'], m
+                                else:
+                                    warnings.warn("!! invalid message " +
+                                                  "received: %r" % msg)
+                        else:
+                            yield _name, ep, _topic, msg
                     else:
                         # Else.. we are supposed to be validating, but the
                         # message failed validation.
